import z from 'zod';
import { PluginAPI } from './plugin-api';
import { PluginMetadata } from './plugin-metadata';
import { RuleCont } from './rules';
export declare const PLUGIN_DEFAULT_RULES = "@midnight-smoker/plugin-default-rules";
/**
 * A plugin as stored internally
 */
type RegisteredPlugin = {
    plugin: unknown;
    metadata: PluginMetadata;
};
/**
 * Schema for return value of the {@link zPluginFactory} function
 */
declare const zPluginDefinition: z.ZodObject<{
    rules: z.ZodDefault<z.ZodArray<z.ZodObject<{
        name: z.ZodString;
        description: z.ZodString;
        defaultSeverity: z.ZodOptional<z.ZodEnum<["off", "warn", "error"]>>;
        check: z.ZodFunction<z.ZodTuple<[], z.ZodUnknown>, z.ZodUnknown>;
        schema: z.ZodOptional<z.ZodType<z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>, z.ZodTypeDef, z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>>>;
    }, "strict", z.ZodTypeAny, {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }, {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }>, "many">>;
}, "strip", z.ZodTypeAny, {
    rules: {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }[];
}, {
    rules?: {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }[] | undefined;
}>;
/**
 * The return value of a {@link zPluginFactory} function
 */
export type PluginDefinition = z.infer<typeof zPluginDefinition>;
declare const zPluginFactory: z.ZodFunction<z.ZodTuple<[z.ZodAny], null>, z.ZodObject<{
    rules: z.ZodDefault<z.ZodArray<z.ZodObject<{
        name: z.ZodString;
        description: z.ZodString;
        defaultSeverity: z.ZodOptional<z.ZodEnum<["off", "warn", "error"]>>;
        check: z.ZodFunction<z.ZodTuple<[], z.ZodUnknown>, z.ZodUnknown>;
        schema: z.ZodOptional<z.ZodType<z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>, z.ZodTypeDef, z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>>>;
    }, "strict", z.ZodTypeAny, {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }, {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }>, "many">>;
}, "strip", z.ZodTypeAny, {
    rules: {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }[];
}, {
    rules?: {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }[] | undefined;
}>>;
export type PluginFactory = z.infer<typeof zPluginFactory>;
declare const zPluginObject: z.ZodObject<{
    plugin: z.ZodFunction<z.ZodTuple<[z.ZodAny], null>, z.ZodObject<{
        rules: z.ZodDefault<z.ZodArray<z.ZodObject<{
            name: z.ZodString;
            description: z.ZodString;
            defaultSeverity: z.ZodOptional<z.ZodEnum<["off", "warn", "error"]>>;
            check: z.ZodFunction<z.ZodTuple<[], z.ZodUnknown>, z.ZodUnknown>;
            schema: z.ZodOptional<z.ZodType<z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>, z.ZodTypeDef, z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>>>;
        }, "strict", z.ZodTypeAny, {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }, {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }>, "many">>;
    }, "strip", z.ZodTypeAny, {
        rules: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[];
    }, {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }>>;
}, "strip", z.ZodTypeAny, {
    plugin: (args_0: any) => {
        rules: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[];
    };
}, {
    plugin: (args_0: any) => {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    };
}>;
export type PluginObject = z.infer<typeof zPluginObject>;
declare const zPlugin: z.ZodPipeline<z.ZodUnion<[z.ZodObject<{
    plugin: z.ZodFunction<z.ZodTuple<[z.ZodAny], null>, z.ZodObject<{
        rules: z.ZodDefault<z.ZodArray<z.ZodObject<{
            name: z.ZodString;
            description: z.ZodString;
            defaultSeverity: z.ZodOptional<z.ZodEnum<["off", "warn", "error"]>>;
            check: z.ZodFunction<z.ZodTuple<[], z.ZodUnknown>, z.ZodUnknown>;
            schema: z.ZodOptional<z.ZodType<z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>, z.ZodTypeDef, z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>>>;
        }, "strict", z.ZodTypeAny, {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }, {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }>, "many">>;
    }, "strip", z.ZodTypeAny, {
        rules: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[];
    }, {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }>>;
}, "strip", z.ZodTypeAny, {
    plugin: (args_0: any) => {
        rules: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[];
    };
}, {
    plugin: (args_0: any) => {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    };
}>, z.ZodEffects<z.ZodObject<{
    default: z.ZodUnion<[z.ZodObject<{
        plugin: z.ZodFunction<z.ZodTuple<[z.ZodAny], null>, z.ZodObject<{
            rules: z.ZodDefault<z.ZodArray<z.ZodObject<{
                name: z.ZodString;
                description: z.ZodString;
                defaultSeverity: z.ZodOptional<z.ZodEnum<["off", "warn", "error"]>>;
                check: z.ZodFunction<z.ZodTuple<[], z.ZodUnknown>, z.ZodUnknown>;
                schema: z.ZodOptional<z.ZodType<z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>, z.ZodTypeDef, z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>>>;
            }, "strict", z.ZodTypeAny, {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }, {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }>, "many">>;
        }, "strip", z.ZodTypeAny, {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        }, {
            rules?: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[] | undefined;
        }>>;
    }, "strip", z.ZodTypeAny, {
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    }, {
        plugin: (args_0: any) => {
            rules?: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[] | undefined;
        };
    }>, z.ZodEffects<z.ZodFunction<z.ZodTuple<[z.ZodAny], null>, z.ZodObject<{
        rules: z.ZodDefault<z.ZodArray<z.ZodObject<{
            name: z.ZodString;
            description: z.ZodString;
            defaultSeverity: z.ZodOptional<z.ZodEnum<["off", "warn", "error"]>>;
            check: z.ZodFunction<z.ZodTuple<[], z.ZodUnknown>, z.ZodUnknown>;
            schema: z.ZodOptional<z.ZodType<z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>, z.ZodTypeDef, z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>>>;
        }, "strict", z.ZodTypeAny, {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }, {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }>, "many">>;
    }, "strip", z.ZodTypeAny, {
        rules: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[];
    }, {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }>>, {
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    }, (args_0: any) => {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }>]>;
}, "strip", z.ZodTypeAny, {
    default: ({
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    } | {
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    }) & ({
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    } | {
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    } | undefined);
}, {
    default: (((args_0: any) => {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }) | {
        plugin: (args_0: any) => {
            rules?: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[] | undefined;
        };
    }) & (((args_0: any) => {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }) | {
        plugin: (args_0: any) => {
            rules?: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[] | undefined;
        };
    } | undefined);
}>, {
    plugin: ({
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    } | {
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    }) & ({
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    } | {
        plugin: (args_0: any) => {
            rules: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[];
        };
    } | undefined);
}, {
    default: (((args_0: any) => {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }) | {
        plugin: (args_0: any) => {
            rules?: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[] | undefined;
        };
    }) & (((args_0: any) => {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }) | {
        plugin: (args_0: any) => {
            rules?: {
                description: string;
                name: string;
                check: (...args: unknown[]) => unknown;
                defaultSeverity?: "error" | "warn" | "off" | undefined;
                schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
            }[] | undefined;
        };
    } | undefined);
}>, z.ZodEffects<z.ZodFunction<z.ZodTuple<[z.ZodAny], null>, z.ZodObject<{
    rules: z.ZodDefault<z.ZodArray<z.ZodObject<{
        name: z.ZodString;
        description: z.ZodString;
        defaultSeverity: z.ZodOptional<z.ZodEnum<["off", "warn", "error"]>>;
        check: z.ZodFunction<z.ZodTuple<[], z.ZodUnknown>, z.ZodUnknown>;
        schema: z.ZodOptional<z.ZodType<z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>, z.ZodTypeDef, z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>>>;
    }, "strict", z.ZodTypeAny, {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }, {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }>, "many">>;
}, "strip", z.ZodTypeAny, {
    rules: {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }[];
}, {
    rules?: {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }[] | undefined;
}>>, {
    plugin: (args_0: any) => {
        rules: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[];
    };
}, (args_0: any) => {
    rules?: {
        description: string;
        name: string;
        check: (...args: unknown[]) => unknown;
        defaultSeverity?: "error" | "warn" | "off" | undefined;
        schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
    }[] | undefined;
}>]>, z.ZodObject<{
    plugin: z.ZodFunction<z.ZodTuple<[z.ZodAny], null>, z.ZodObject<{
        rules: z.ZodDefault<z.ZodArray<z.ZodObject<{
            name: z.ZodString;
            description: z.ZodString;
            defaultSeverity: z.ZodOptional<z.ZodEnum<["off", "warn", "error"]>>;
            check: z.ZodFunction<z.ZodTuple<[], z.ZodUnknown>, z.ZodUnknown>;
            schema: z.ZodOptional<z.ZodType<z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>, z.ZodTypeDef, z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown>>>;
        }, "strict", z.ZodTypeAny, {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }, {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }>, "many">>;
    }, "strip", z.ZodTypeAny, {
        rules: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[];
    }, {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    }>>;
}, "strip", z.ZodTypeAny, {
    plugin: (args_0: any) => {
        rules: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[];
    };
}, {
    plugin: (args_0: any) => {
        rules?: {
            description: string;
            name: string;
            check: (...args: unknown[]) => unknown;
            defaultSeverity?: "error" | "warn" | "off" | undefined;
            schema?: z.ZodObject<z.ZodRawShape, z.UnknownKeysParam, z.ZodTypeAny, unknown, unknown> | undefined;
        }[] | undefined;
    };
}>>;
/**
 * The shape a plugin entry point can assume. A module exporting a named
 * `plugin` {@link PluginFactory}, or the default export being a
 * {@link PluginFactory}.
 */
export type Plugin = z.infer<typeof zPlugin>;
export declare class PluginRegistry extends Map<string, RegisteredPlugin> {
    private constructor();
    static create(entries?: readonly (readonly [string, RegisteredPlugin])[] | null): PluginRegistry;
    getCheckOptions(): z.ZodDefault<z.ZodObject<{}, "strip", z.ZodPipeline<z.ZodUnion<[z.ZodEffects<z.ZodDefault<z.ZodObject<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, "strict", z.ZodTypeAny, {
        [x: string]: any;
        [x: number]: any;
    }, {
        [x: string]: any;
        [x: number]: any;
    }>>, {
        severity: "error" | "warn" | "off";
        opts: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">;
    }, {
        [x: string]: any;
        [x: number]: any;
    } | undefined> | z.ZodEffects<z.ZodDefault<z.ZodObject<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, "passthrough", z.ZodTypeAny, z.objectOutputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough">, z.objectInputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough">>>, {
        severity: "error" | "warn" | "off";
        opts: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">;
    }, z.objectInputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough"> | undefined>, z.ZodEffects<z.ZodDefault<z.ZodEnum<["off", "warn", "error"]>>, {
        severity: "error" | "warn" | "off";
        opts: Record<string, any>;
    }, "error" | "warn" | "off" | undefined>, z.ZodEffects<z.ZodTuple<[z.ZodDefault<z.ZodEnum<["off", "warn", "error"]>>, z.ZodDefault<z.ZodObject<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, "passthrough", z.ZodTypeAny, z.objectOutputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough">, z.objectInputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough">>> | z.ZodDefault<z.ZodObject<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, "strict", z.ZodTypeAny, {
        [x: string]: any;
        [x: number]: any;
    }, {
        [x: string]: any;
        [x: number]: any;
    }>>], null>, {
        severity: "error" | "warn" | "off";
        opts: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">;
    }, ["error" | "warn" | "off" | undefined, {
        [x: string]: any;
        [x: number]: any;
    } | z.objectInputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough"> | undefined]>, z.ZodObject<{
        severity: z.ZodDefault<z.ZodEnum<["off", "warn", "error"]>>;
        opts: z.ZodDefault<z.ZodObject<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, "passthrough", z.ZodTypeAny, z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">, z.objectInputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">>> | z.ZodDefault<z.ZodObject<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, "strict", z.ZodTypeAny, {
            [x: string]: any;
            [x: number]: any;
        }, {
            [x: string]: any;
            [x: number]: any;
        }>>;
    }, "strict", z.ZodTypeAny, {
        opts: ({
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">) & ({
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough"> | undefined);
        severity: "error" | "warn" | "off";
    }, {
        severity?: "error" | "warn" | "off" | undefined;
        opts?: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectInputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough"> | undefined;
    }>]>, z.ZodObject<{
        severity: z.ZodDefault<z.ZodEnum<["off", "warn", "error"]>>;
        opts: z.ZodDefault<z.ZodObject<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, "passthrough", z.ZodTypeAny, z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">, z.objectInputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">>> | z.ZodDefault<z.ZodObject<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, "strict", z.ZodTypeAny, {
            [x: string]: any;
            [x: number]: any;
        }, {
            [x: string]: any;
            [x: number]: any;
        }>>;
    }, "strict", z.ZodTypeAny, {
        opts: ({
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">) & ({
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough"> | undefined);
        severity: "error" | "warn" | "off";
    }, {
        severity?: "error" | "warn" | "off" | undefined;
        opts?: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectInputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough"> | undefined;
    }>>, {}, {}>>;
    getRuleConts(): RuleCont[];
    /**
     *
     * @param cwd Current working directory
     * @returns A function that will unload all plugins
     */
    loadPlugins(cwd?: string): Promise<this>;
    loadPlugins(pluginNames: string[], cwd?: string): Promise<this>;
    mergeRuleDefaults(): Record<string, unknown>;
    mergeRuleSchemas(): z.ZodObject<{}, "strip", z.ZodPipeline<z.ZodUnion<[z.ZodEffects<z.ZodDefault<z.ZodObject<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, "strict", z.ZodTypeAny, {
        [x: string]: any;
        [x: number]: any;
    }, {
        [x: string]: any;
        [x: number]: any;
    }>>, {
        severity: "error" | "warn" | "off";
        opts: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">;
    }, {
        [x: string]: any;
        [x: number]: any;
    } | undefined> | z.ZodEffects<z.ZodDefault<z.ZodObject<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, "passthrough", z.ZodTypeAny, z.objectOutputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough">, z.objectInputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough">>>, {
        severity: "error" | "warn" | "off";
        opts: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">;
    }, z.objectInputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough"> | undefined>, z.ZodEffects<z.ZodDefault<z.ZodEnum<["off", "warn", "error"]>>, {
        severity: "error" | "warn" | "off";
        opts: Record<string, any>;
    }, "error" | "warn" | "off" | undefined>, z.ZodEffects<z.ZodTuple<[z.ZodDefault<z.ZodEnum<["off", "warn", "error"]>>, z.ZodDefault<z.ZodObject<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, "passthrough", z.ZodTypeAny, z.objectOutputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough">, z.objectInputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough">>> | z.ZodDefault<z.ZodObject<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, "strict", z.ZodTypeAny, {
        [x: string]: any;
        [x: number]: any;
    }, {
        [x: string]: any;
        [x: number]: any;
    }>>], null>, {
        severity: "error" | "warn" | "off";
        opts: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">;
    }, ["error" | "warn" | "off" | undefined, {
        [x: string]: any;
        [x: number]: any;
    } | z.objectInputType<Omit<{
        [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
    }, "opts">, z.ZodTypeAny, "passthrough"> | undefined]>, z.ZodObject<{
        severity: z.ZodDefault<z.ZodEnum<["off", "warn", "error"]>>;
        opts: z.ZodDefault<z.ZodObject<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, "passthrough", z.ZodTypeAny, z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">, z.objectInputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">>> | z.ZodDefault<z.ZodObject<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, "strict", z.ZodTypeAny, {
            [x: string]: any;
            [x: number]: any;
        }, {
            [x: string]: any;
            [x: number]: any;
        }>>;
    }, "strict", z.ZodTypeAny, {
        opts: ({
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">) & ({
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough"> | undefined);
        severity: "error" | "warn" | "off";
    }, {
        severity?: "error" | "warn" | "off" | undefined;
        opts?: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectInputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough"> | undefined;
    }>]>, z.ZodObject<{
        severity: z.ZodDefault<z.ZodEnum<["off", "warn", "error"]>>;
        opts: z.ZodDefault<z.ZodObject<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, "passthrough", z.ZodTypeAny, z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">, z.objectInputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">>> | z.ZodDefault<z.ZodObject<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, "strict", z.ZodTypeAny, {
            [x: string]: any;
            [x: number]: any;
        }, {
            [x: string]: any;
            [x: number]: any;
        }>>;
    }, "strict", z.ZodTypeAny, {
        opts: ({
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough">) & ({
            [x: string]: any;
            [x: number]: any;
        } | z.objectOutputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough"> | undefined);
        severity: "error" | "warn" | "off";
    }, {
        severity?: "error" | "warn" | "off" | undefined;
        opts?: {
            [x: string]: any;
            [x: number]: any;
        } | z.objectInputType<Omit<{
            [x: string]: z.ZodOptional<any> | z.ZodDefault<any>;
        }, "opts">, z.ZodTypeAny, "passthrough"> | undefined;
    }>>, {}, {}>;
    registerPlugin(entryPoint: string, name?: string): Promise<unknown>;
    registerPlugin(metadata: PluginMetadata): Promise<unknown>;
    protected static resolvePlugin(pluginName: string, cwd?: string): Promise<PluginMetadata>;
    protected createPluginAPI(metadata: PluginMetadata): PluginAPI;
    private static normalizePlugin;
}
export type ScopedRuleName<PName extends string, RuleName extends string> = `${PName}/${RuleName}`;
export {};
//# sourceMappingURL=plugin-registry.d.ts.map