"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginRegistry = exports.PLUGIN_DEFAULT_RULES = void 0;
// import * as pluginDefaultRules from '@midnight-smoker/plugin-default-rules';
const debug_1 = __importDefault(require("debug"));
const path_1 = require("path");
const zod_1 = __importDefault(require("zod"));
const zod_validation_error_1 = require("zod-validation-error");
const plugin_api_1 = require("./plugin-api");
const plugin_metadata_1 = require("./plugin-metadata");
const rules_1 = require("./rules");
const schemaUtils = __importStar(require("./schema-util"));
const util_1 = require("./util");
exports.PLUGIN_DEFAULT_RULES = '@midnight-smoker/plugin-default-rules';
const debug = (0, debug_1.default)('midnight-smoker:plugin-registry');
/**
 * Rough sketch of what a {@link RuleDef} looks like
 */
const zBaseRuleDef = zod_1.default.strictObject({
    name: zod_1.default.string().min(1),
    description: zod_1.default.string().min(1),
    defaultSeverity: rules_1.zCheckSeverity.optional(),
    check: zod_1.default.function(),
    schema: zod_1.default.instanceof(zod_1.default.ZodObject).optional(),
});
/**
 * Schema for return value of the {@link zPluginFactory} function
 */
const zPluginDefinition = zod_1.default.object({
    rules: zod_1.default.array(zBaseRuleDef).default([]),
});
const zPluginFactory = zod_1.default
    .function(zod_1.default.tuple([zod_1.default.any()]))
    .returns(zPluginDefinition);
const zPluginObject = zod_1.default.object({
    plugin: zPluginFactory,
});
const zNormalizedPluginFactory = zPluginFactory.transform((value) => ({
    plugin: value,
}));
const zESMSucks = zod_1.default
    .object({
    default: zPluginObject.or(zNormalizedPluginFactory),
})
    .transform((value) => ({ plugin: value.default }));
const zPlugin = zod_1.default
    .union([zPluginObject, zESMSucks, zNormalizedPluginFactory])
    .pipe(zPluginObject);
class PluginRegistry extends Map {
    constructor(entries) {
        super(entries);
    }
    static create(entries) {
        return new PluginRegistry(entries);
    }
    getCheckOptions() {
        const zMergedSchemas = this.mergeRuleSchemas();
        return zMergedSchemas
            .describe('Rule configuration for automated checks')
            .default(this.mergeRuleDefaults());
    }
    getRuleConts() {
        return [...this.values()].flatMap(({ metadata }) => [
            ...metadata.ruleConts.values(),
        ]);
    }
    async loadPlugins(pluginNamesOrCwd, cwd = process.cwd()) {
        if (typeof pluginNamesOrCwd === 'string') {
            cwd = pluginNamesOrCwd;
            pluginNamesOrCwd = [];
        }
        else if (!Array.isArray(pluginNamesOrCwd)) {
            pluginNamesOrCwd = [];
        }
        const pluginNames = [
            ...pluginNamesOrCwd,
            exports.PLUGIN_DEFAULT_RULES,
        ];
        await Promise.all(pluginNames.map((plugin) => PluginRegistry.resolvePlugin(plugin, cwd)
            .then((metadata) => this.registerPlugin(metadata))
            .catch((err) => {
            if (err instanceof zod_1.default.ZodError || err.name === 'ZodError') {
                throw (0, zod_validation_error_1.fromZodError)(err);
            }
            throw err;
        })));
        return this;
    }
    mergeRuleDefaults() {
        return [...this.values()].reduce((defaults, { metadata }) => [...metadata.ruleConts.values()].reduce((acc, rc) => rc((rule) => ({
            ...acc,
            [rule.name]: rule.defaultOptions,
        })), defaults), {});
    }
    mergeRuleSchemas() {
        return [...this.values()].reduce((zPluginRuleSchema, { metadata }) => zPluginRuleSchema.merge([...metadata.ruleConts.values()].reduce((zRuleSchema, rc) => rc((rule) => zRuleSchema.setKey(rule.name, rule.zRuleSchema)), zod_1.default.object({}).catchall(rules_1.zBaseRuleOptions))), zod_1.default.object({}).catchall(rules_1.zBaseRuleOptions));
    }
    async registerPlugin(metadata, name) {
        if (typeof metadata === 'string') {
            metadata = new plugin_metadata_1.PluginMetadata(metadata, name);
        }
        if (this.has(metadata.name)) {
            throw new Error(`Plugin with name "${metadata.name}" already loaded`);
        }
        if (metadata.name === exports.PLUGIN_DEFAULT_RULES) {
            const resolvedDefaultRules = require.resolve(exports.PLUGIN_DEFAULT_RULES);
            if (resolvedDefaultRules !== metadata.entryPoint) {
                throw new Error(`${exports.PLUGIN_DEFAULT_RULES} is not the expected module; expected ${resolvedDefaultRules} but got ${metadata.entryPoint}`);
            }
        }
        let rawPlugin;
        debug('Attempting to load plugin %s', metadata);
        try {
            rawPlugin = require(metadata.entryPoint);
            debug('Loaded plugin %s', metadata);
        }
        catch (err) {
            throw new Error(`Plugin ${metadata} failed to load: ${err.message}`);
        }
        // eslint-disable-next-line @typescript-eslint/no-var-requires
        const plugin = PluginRegistry.normalizePlugin(rawPlugin);
        const pluginApi = this.createPluginAPI(metadata);
        try {
            const pluginInstance = plugin.plugin(pluginApi);
            const { ruleConts } = metadata;
            for (const ruleDef of pluginInstance.rules) {
                const rule = new rules_1.Rule(ruleDef);
                ruleConts.set(rule.name, rule.toRuleCont());
            }
            debug('Found %d rule(s) in plugin %s', ruleConts.size, metadata);
            this.set(metadata.name, {
                plugin: pluginInstance,
                metadata,
            });
            return pluginInstance;
        }
        catch (err) {
            throw new Error(`Plugin ${metadata} failed to initialize: ${err.message}`);
        }
    }
    static async resolvePlugin(pluginName, cwd = __dirname) {
        let entryPoint;
        try {
            entryPoint = (0, util_1.resolveFrom)(pluginName, cwd);
        }
        catch (err) {
            if (err.code !== 'MODULE_NOT_FOUND') {
                throw err;
            }
            try {
                cwd = process.cwd();
                entryPoint = (0, util_1.resolveFrom)(pluginName, cwd);
            }
            catch (err) {
                if (err.code !== 'MODULE_NOT_FOUND') {
                    throw err;
                }
                throw new Error(`Could not find plugin ${pluginName}`);
            }
        }
        debug('Found entry point %s for plugin %s', entryPoint, pluginName);
        const pkgResult = await (0, util_1.readPackageJson)({
            cwd: (0, path_1.dirname)(entryPoint),
            normalize: true,
            strict: true,
        });
        if (!pkgResult) {
            throw new Error(`Could not find package.json from ${cwd}`);
        }
        const { packageJson: { name, version }, } = pkgResult;
        const metadata = new plugin_metadata_1.PluginMetadata({
            name,
            version,
            requestedAs: pluginName,
            entryPoint,
        });
        return Object.freeze(metadata);
    }
    createPluginAPI(metadata) {
        return {
            schemaUtils,
            metadata,
            z: zod_1.default,
            zod: zod_1.default,
            activePlugins: [...this.values()].map(({ metadata }) => metadata),
            defineRule: plugin_api_1.defineRule,
        };
    }
    static normalizePlugin(plugin) {
        return zPlugin.parse(plugin);
    }
}
exports.PluginRegistry = PluginRegistry;
//# sourceMappingURL=plugin-registry.js.map