import type { PackageJson } from 'read-pkg-up';
import { CheckContextFailFn, Rule, RuleOptionSchema, StaticCheckContext } from './rule';
import { type CheckSeverity } from './severity';
/**
 * A context object which is provided to a {@linkcode RuleCheckFn}, containing
 * information about the current package to be checked and how to report a
 * failure.
 *
 * @public
 */
export declare class CheckContext<const Name extends string = string, Schema extends RuleOptionSchema | void = void> implements StaticCheckContext {
    private readonly staticCtx;
    /**
     * The (parse as )d `package.json` for the package being checked.
     *
     * _Not_ normalized.
     */
    readonly pkgJson: PackageJson;
    /**
     * The absolute path to this context's package's `package.json`.
     */
    readonly pkgJsonPath: string;
    /**
     * The absolute path to this context's package's root directory.
     *
     * This is the same as `path.dirname(pkgJsonPath)`
     */
    readonly pkgPath: string;
    /**
     * The severity level for this rule (as chosen by the user)
     */
    readonly severity: CheckSeverity;
    /**
     * The `fail` function as provided to the `Rule`'s {@linkcode RuleCheckFn}.
     */
    readonly fail: CheckContextFailFn;
    constructor(rule: Rule<Name, Schema>, staticCtx: StaticCheckContext);
    /**
     * Creates a `fail` function for {@linkcode RuleCheckFn}.
     * @param rule The `Rule` which will receive the `fail` function
     * @returns A `fail` function
     */
    private createFailFn;
    /**
     * Omits the {@linkcode CheckContext.pkgJson} property (too big).
     *
     * @returns A JSON-serializable representation of this object
     */
    toJSON(): StaticCheckContext;
}
//# sourceMappingURL=context.d.ts.map