"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CheckContext = void 0;
/**
 * A context object which is provided to a {@linkcode RuleCheckFn}, containing
 * information about the current package to be checked and how to report a
 * failure.
 *
 * @public
 */
class CheckContext {
    constructor(rule, staticCtx) {
        this.staticCtx = staticCtx;
        this.pkgJson = staticCtx.pkgJson;
        this.pkgJsonPath = staticCtx.pkgJsonPath;
        this.pkgPath = staticCtx.pkgPath;
        this.severity = staticCtx.severity;
        this.fail = this.createFailFn(rule);
    }
    /**
     * Creates a `fail` function for {@linkcode RuleCheckFn}.
     * @param rule The `Rule` which will receive the `fail` function
     * @returns A `fail` function
     */
    createFailFn(rule) {
        return (message, data) => ({
            rule: rule.toJSON(),
            message,
            data,
            context: this,
            failed: true,
            severity: this.severity,
        });
    }
    /**
     * Omits the {@linkcode CheckContext.pkgJson} property (too big).
     *
     * @returns A JSON-serializable representation of this object
     */
    toJSON() {
        return this.staticCtx;
    }
}
exports.CheckContext = CheckContext;
//# sourceMappingURL=context.js.map